/*
 * Released under MIT license terms, read LICENSE file
 */

#include <gtk/gtk.h>

#include "psensor.h"


int compute_y(float temp, int mint, int maxt, int width, int height) 
{
	double t = temp - mint;
	return height - ((double)height * ( t / (maxt-mint)));
}

void update_graph(struct psensor** sensors, GtkWidget* w_graph) 
{
	int mint = get_min_temp(sensors);
	int maxt = get_max_temp(sensors);


	int width = w_graph->allocation.width;
	int height = w_graph->allocation.height;

	// start offset of the graph
	// 0..g_off used to print max/min T
	int g_xoff = 50;
	// the width of the graph
	int g_width = width - g_xoff;

	cairo_t *cr = gdk_cairo_create(w_graph->window);

	// white background
	cairo_set_source_rgb(cr, 1, 1, 1);
	cairo_rectangle (cr, 0, 0, width, height);
	cairo_fill (cr);


	// setup dash style
	double dashes[] = {1.0,  /* ink */
                   2.0,  /* skip */
                   1.0,  /* ink */
                   2.0   /* skip*/
                  };
	int    ndash  = sizeof (dashes)/sizeof(dashes[0]);
	cairo_set_dash (cr, dashes, ndash, 0);
	cairo_set_line_width (cr, 1);
	cairo_set_source_rgb(cr, 0.0, 0.0, 0.0);


	// vertical lines representing time steps
	int i=0;	
	for(i = 0; i <= 5; i++) {

		int x =  i * (g_width / 5) + g_xoff;
		cairo_move_to(cr,x,0);
		cairo_line_to(cr,x,height);
		cairo_stroke(cr);
	}

	// horizontal lines
	// draws a line for each 10C step
	for(i = mint; i < maxt; i++) {
		if (i % 10 == 0) {
			int y = compute_y(i, mint, maxt, width, height);
			cairo_move_to(cr,g_xoff,y);
			cairo_line_to(cr,g_xoff+g_width,y);
			cairo_stroke(cr);
		}
	}

	// back to normal line style
	cairo_set_dash (cr, 0, 0, 0);

	// print min and max temp in the canvas
	char strmin[200];
	char strmax[200];
	sprintf(strmin,"+%dC",mint);
	sprintf(strmax,"+%dC",maxt);
	cairo_set_source_rgba(cr, 0.0, 0.0, 0.0, 1.0);
	cairo_text_extents_t te;
	cairo_select_font_face (cr, "sans-serif",
				CAIRO_FONT_SLANT_NORMAL, CAIRO_FONT_WEIGHT_NORMAL);
	cairo_set_font_size (cr, 10.0);
	cairo_text_extents (cr, strmax, &te);
	cairo_move_to(cr,g_xoff - te.width - 5,te.height);
	cairo_show_text(cr, strmax);

	cairo_text_extents (cr, strmin, &te);
	cairo_move_to(cr,g_xoff - te.width - 5,height-te.height/2);
	cairo_show_text(cr, strmin);

	// .. and finaly draws the temperature graphs
	struct psensor** sensor_cur = sensors;
	int c = 0;
	while(*sensor_cur) {
		cairo_set_source_rgba(cr, 
				      (*sensor_cur)->color[0], 
				      (*sensor_cur)->color[1], 
				      (*sensor_cur)->color[2], 
				      0.8);
		c++;
		
		cairo_set_line_width (cr, 1);

		int first = 1;
		int i;

		for(i = 0 ; i < MAX_TEMPS; i++) {
			if (!((*sensor_cur)->temps[i])) 
				continue;				

			int x = i * g_width / MAX_TEMPS + g_xoff;
			int y = compute_y((*sensor_cur)->temps[i],
					  mint, maxt,
					  g_width, height);
			
			if (first) {
				cairo_move_to(cr, x, y);
				first = 0;
			} else {
				cairo_line_to(cr, x, y);
			}

		}
		cairo_stroke(cr);
		sensor_cur++;
	}

	cairo_destroy(cr);
}
