/*
 * Released under MIT license terms, read LICENSE file
 */
#include <stdlib.h>
#include <string.h>

#include <sensors/sensors.h>
#include <sensors/error.h>

#include "psensor.h"

/*
 * Returns minimal stored temperature
 */
int get_min_temp(struct psensor** sensors) 
{
	int m = 0;
	struct psensor** s = sensors;

	while(*s) {
		int i;
		int t;
		for(i=0;i<MAX_TEMPS;i++) {
			t = (*s)->temps[i];
			
			if (t && (t < m || m == 0)) 
				m = t;
		}
		s++;
	}

	return m;
}

/*
 * Returns maximal stored temperature
 */
int get_max_temp(struct psensor** sensors) 
{
	int m = 0;
	struct psensor** s = sensors;

	while(*s) {
		int i;
		int t;
		for(i=0;i<MAX_TEMPS;i++) {
			t = (*s)->temps[i];
			
			if (t > m) 
				m = t;
		}
		s++;
	}

	return m;
}

double get_value(const sensors_chip_name *name,
		 const sensors_subfeature *sub)
{
	double val;
	int err;

	err = sensors_get_value(name, sub->number, &val);
	if (err) {
		fprintf(stderr, "ERROR: Can't get value of subfeature %s: %s\n",
			sub->name, sensors_strerror(err));
		val = 0;
	}
	return val;
}


double get_sensor_temp_input(const sensors_chip_name* chip,
			     const sensors_feature* feature)
{
	const sensors_subfeature *sf;
	
	sf = sensors_get_subfeature(chip, feature,
				    SENSORS_SUBFEATURE_TEMP_INPUT);
	if (sf) 
		return get_value(chip, sf);
	else
		return 0;
}

double get_temp_input(struct psensor* sensor)
{
	return get_sensor_temp_input(sensor->iname,
				     sensor->feature);
}


void update_psensor(struct psensor* sensor) 
{
	int i;
	for(i = 1; i < MAX_TEMPS; i++) 
		sensor->temps[i-1] = sensor->temps[i];
	
	sensor->temps[MAX_TEMPS-1] = get_temp_input(sensor);
}



struct psensor* create_sensor(const sensors_chip_name* chip,
			      const sensors_feature* feature) 
{
	char name[200];
	
	if (sensors_snprintf_chip_name(name, 200, chip) < 0)
		return NULL;

	const sensors_subfeature *sf 
		= sensors_get_subfeature(chip, feature,
					 SENSORS_SUBFEATURE_TEMP_FAULT);
	if (sf && get_value(chip, sf)) 
		return NULL;

	
	char *label;				
	if (!(label = sensors_get_label(chip, feature))) 
		return NULL;

	if (!get_sensor_temp_input(chip,feature)) 
		return NULL;

	
	struct psensor* psensor 
		= (struct psensor*)malloc(sizeof(struct psensor));
	psensor->chip_name = strdup(name);
	psensor->name = strdup(label);
	psensor->iname = chip;
	psensor->feature = feature;

	int t;
	for(t=0;t<MAX_TEMPS;t++) 
		psensor->temps[t] = 0;

	return psensor;
}
